package org.eclipse.jdt.internal.compiler.batch;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.DefaultErrorHandlingPolicies;
import org.eclipse.jdt.internal.compiler.IProblemFactory;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.batch.ClasspathLocation;
import org.eclipse.jdt.internal.compiler.batch.CompilationUnit;
import org.eclipse.jdt.internal.compiler.batch.FileSystem;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFileReader;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFormatException;
import org.eclipse.jdt.internal.compiler.env.AccessRuleSet;
import org.eclipse.jdt.internal.compiler.env.IBinaryType;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.env.NameEnvironmentAnswer;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.parser.Parser;
import org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.util.Util;

public class ClasspathDirectory extends ClasspathLocation {
  private Hashtable directoryCache;
  
  private String[] missingPackageHolder = new String[1];
  
  private int mode;
  
  private String encoding;
  
  private Hashtable<String, Hashtable<String, String>> packageSecondaryTypes = null;
  
  Map options;
  
  ClasspathDirectory(File directory, String encoding, int mode, AccessRuleSet accessRuleSet, String destinationPath, Map options) {
    super(accessRuleSet, destinationPath);
    this.mode = mode;
    this.options = options;
    try {
      this.path = directory.getCanonicalPath();
    } catch (IOException iOException) {
      this.path = directory.getAbsolutePath();
    } 
    if (!this.path.endsWith(File.separator))
      this.path = String.valueOf(this.path) + File.separator; 
    this.directoryCache = new Hashtable<>(11);
    this.encoding = encoding;
  }
  
  String[] directoryList(String qualifiedPackageName) {
    String[] dirList = (String[])this.directoryCache.get(qualifiedPackageName);
    if (dirList == this.missingPackageHolder)
      return null; 
    if (dirList != null)
      return dirList; 
    File dir = new File(String.valueOf(this.path) + qualifiedPackageName);
    if (dir.isDirectory()) {
      int index = qualifiedPackageName.length();
      int last = qualifiedPackageName.lastIndexOf(File.separatorChar);
      do {
      
      } while (--index > last && !ScannerHelper.isUpperCase(qualifiedPackageName.charAt(index)));
      if (index > last)
        if (last == -1) {
          if (!doesFileExist(qualifiedPackageName, Util.EMPTY_STRING)) {
            this.directoryCache.put(qualifiedPackageName, this.missingPackageHolder);
            return null;
          } 
        } else {
          String packageName = qualifiedPackageName.substring(last + 1);
          String parentPackage = qualifiedPackageName.substring(0, last);
          if (!doesFileExist(packageName, parentPackage)) {
            this.directoryCache.put(qualifiedPackageName, this.missingPackageHolder);
            return null;
          } 
        }  
      if ((dirList = dir.list()) == null)
        dirList = CharOperation.NO_STRINGS; 
      this.directoryCache.put(qualifiedPackageName, dirList);
      return dirList;
    } 
    this.directoryCache.put(qualifiedPackageName, this.missingPackageHolder);
    return null;
  }
  
  boolean doesFileExist(String fileName, String qualifiedPackageName) {
    String[] dirList = directoryList(qualifiedPackageName);
    if (dirList == null)
      return false; 
    for (int i = dirList.length; --i >= 0;) {
      if (fileName.equals(dirList[i]))
        return true; 
    } 
    return false;
  }
  
  public List fetchLinkedJars(FileSystem.ClasspathSectionProblemReporter problemReporter) {
    return null;
  }
  
  public NameEnvironmentAnswer findClass(char[] typeName, String qualifiedPackageName, String qualifiedBinaryFileName) {
    return findClass(typeName, qualifiedPackageName, qualifiedBinaryFileName, false);
  }
  
  public NameEnvironmentAnswer findClass(char[] typeName, String qualifiedPackageName, String qualifiedBinaryFileName, boolean asBinaryOnly) {
    if (!isPackage(qualifiedPackageName))
      return null; 
    String fileName = new String(typeName);
    boolean binaryExists = ((this.mode & 0x2) != 0 && doesFileExist(String.valueOf(fileName) + ".class", qualifiedPackageName));
    boolean sourceExists = ((this.mode & 0x1) != 0 && doesFileExist(String.valueOf(fileName) + ".java", qualifiedPackageName));
    if (sourceExists && !asBinaryOnly) {
      String fullSourcePath = String.valueOf(this.path) + qualifiedBinaryFileName.substring(0, qualifiedBinaryFileName.length() - 6) + ".java";
      if (!binaryExists)
        return new NameEnvironmentAnswer((ICompilationUnit)new CompilationUnit(null, 
              fullSourcePath, this.encoding, this.destinationPath), 
            fetchAccessRestriction(qualifiedBinaryFileName)); 
      String fullBinaryPath = String.valueOf(this.path) + qualifiedBinaryFileName;
      long binaryModified = (new File(fullBinaryPath)).lastModified();
      long sourceModified = (new File(fullSourcePath)).lastModified();
      if (sourceModified > binaryModified)
        return new NameEnvironmentAnswer((ICompilationUnit)new CompilationUnit(null, 
              fullSourcePath, this.encoding, this.destinationPath), 
            fetchAccessRestriction(qualifiedBinaryFileName)); 
    } 
    if (binaryExists)
      try {
        ClassFileReader reader = ClassFileReader.read(String.valueOf(this.path) + qualifiedBinaryFileName);
        String typeSearched = (qualifiedPackageName.length() > 0) ? (
          String.valueOf(qualifiedPackageName.replace(File.separatorChar, '/')) + "/" + fileName) : 
          fileName;
        if (!CharOperation.equals(reader.getName(), typeSearched.toCharArray()))
          reader = null; 
        if (reader != null)
          return new NameEnvironmentAnswer(
              (IBinaryType)reader, 
              fetchAccessRestriction(qualifiedBinaryFileName)); 
      } catch (IOException iOException) {
      
      } catch (ClassFormatException classFormatException) {} 
    return null;
  }
  
  public NameEnvironmentAnswer findSecondaryInClass(char[] typeName, String qualifiedPackageName, String qualifiedBinaryFileName) {
    if (TypeConstants.PACKAGE_INFO_NAME.equals(typeName))
      return null; 
    String typeNameString = new String(typeName);
    boolean prereqs = (this.options != null && isPackage(qualifiedPackageName) && (this.mode & 0x1) != 0 && doesFileExist(String.valueOf(typeNameString) + ".java", qualifiedPackageName));
    return prereqs ? null : findSourceSecondaryType(typeNameString, qualifiedPackageName, qualifiedBinaryFileName);
  }
  
  public boolean hasAnnotationFileFor(String qualifiedTypeName) {
    int pos = qualifiedTypeName.lastIndexOf('/');
    if (pos != -1 && pos + 1 < qualifiedTypeName.length()) {
      String fileName = String.valueOf(qualifiedTypeName.substring(pos + 1)) + ".eea";
      return doesFileExist(fileName, qualifiedTypeName.substring(0, pos));
    } 
    return false;
  }
  
  private Hashtable<String, String> getPackageTypes(String qualifiedPackageName) {
    Hashtable<String, String> packageEntry = new Hashtable<>();
    String[] dirList = (String[])this.directoryCache.get(qualifiedPackageName);
    if (dirList == this.missingPackageHolder || 
      dirList == null)
      return packageEntry; 
    File dir = new File(String.valueOf(this.path) + qualifiedPackageName);
    File[] listFiles = dir.isDirectory() ? dir.listFiles() : null;
    if (listFiles == null)
      return packageEntry; 
    for (int i = 0, l = listFiles.length; i < l; i++) {
      File f = listFiles[i];
      if (!f.isDirectory()) {
        String s = f.getAbsolutePath();
        if (s != null && (
          s.endsWith(".java") || s.endsWith(".JAVA"))) {
          CompilationUnit cu = new CompilationUnit(null, s, this.encoding, this.destinationPath);
          CompilationResult compilationResult = new CompilationResult(cu.getContents(), 1, 1, 10);
          ProblemReporter problemReporter = 
            new ProblemReporter(
              DefaultErrorHandlingPolicies.proceedWithAllProblems(), 
              new CompilerOptions(this.options), 
              (IProblemFactory)new DefaultProblemFactory());
          Parser parser = new Parser(problemReporter, false);
          parser.reportSyntaxErrorIsRequired = false;
          CompilationUnitDeclaration unit = parser.parse((ICompilationUnit)cu, compilationResult);
          TypeDeclaration[] types = (unit != null) ? unit.types : null;
          if (types != null)
            for (int j = 0, k = types.length; j < k; j++) {
              TypeDeclaration type = types[j];
              char[] name = type.isSecondary() ? type.name : null;
              if (name != null)
                packageEntry.put(new String(name), s); 
            }  
        } 
      } 
    } 
    return packageEntry;
  }
  
  private NameEnvironmentAnswer findSourceSecondaryType(String typeName, String qualifiedPackageName, String qualifiedBinaryFileName) {
    if (this.packageSecondaryTypes == null)
      this.packageSecondaryTypes = new Hashtable<>(); 
    Hashtable<String, String> packageEntry = this.packageSecondaryTypes.get(qualifiedPackageName);
    if (packageEntry == null) {
      packageEntry = getPackageTypes(qualifiedPackageName);
      this.packageSecondaryTypes.put(qualifiedPackageName, packageEntry);
    } 
    String fileName = packageEntry.get(typeName);
    return (fileName != null) ? new NameEnvironmentAnswer((ICompilationUnit)new CompilationUnit(null, 
          fileName, this.encoding, this.destinationPath), 
        fetchAccessRestriction(qualifiedBinaryFileName)) : null;
  }
  
  public char[][][] findTypeNames(String qualifiedPackageName) {
    if (!isPackage(qualifiedPackageName))
      return null; 
    File dir = new File(String.valueOf(this.path) + qualifiedPackageName);
    if (!dir.exists() || !dir.isDirectory())
      return null; 
    String[] listFiles = dir.list((FilenameFilter)new Object(this));
    int length;
    if (listFiles == null || (length = listFiles.length) == 0)
      return null; 
    char[][][] result = new char[length][][];
    char[][] packageName = CharOperation.splitOn(File.separatorChar, qualifiedPackageName.toCharArray());
    for (int i = 0; i < length; i++) {
      String fileName = listFiles[i];
      int indexOfLastDot = fileName.indexOf('.');
      result[i] = CharOperation.arrayConcat(packageName, fileName.substring(0, indexOfLastDot).toCharArray());
    } 
    return result;
  }
  
  public void initialize() throws IOException {}
  
  public boolean isPackage(String qualifiedPackageName) {
    return (directoryList(qualifiedPackageName) != null);
  }
  
  public void reset() {
    this.directoryCache = new Hashtable<>(11);
  }
  
  public String toString() {
    return "ClasspathDirectory " + this.path;
  }
  
  public char[] normalizedPath() {
    if (this.normalizedPath == null) {
      this.normalizedPath = this.path.toCharArray();
      if (File.separatorChar == '\\')
        CharOperation.replace(this.normalizedPath, '\\', '/'); 
    } 
    return this.normalizedPath;
  }
  
  public String getPath() {
    return this.path;
  }
  
  public int getMode() {
    return this.mode;
  }
}
